package com.njyunzhi.servsummary.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.njyunzhi.servsummary.entity.SysSms;
import com.njyunzhi.servsummary.service.SysSmsService;
import lombok.Data;
import lombok.experimental.Accessors;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.Hashtable;
import java.util.Random;

@Slf4j
@Component
public class SMSCaptcha {

    @Value("${yz.sms.captcha.code}")
    String code;
    @Value("${yz.sms.captcha.sign}")
    String sign;

    @Autowired
    SMSUtils smsUtils;

    // 数据库有定时任务, 定时清理过期短信
    @Autowired
    SysSmsService sysSmsService;

    // 默认超时时间, 单位秒
    Integer expireSec = 15 * 60;

    /**
     * 发送验证码
     * @param tel
     * @param captcha
     * @throws Exception
     */
    public void send(String tel, String captcha) throws Exception {
        if (null == captcha || "".equals(captcha)) {
            throw new Exception("没有有效的验证码");
        }

        SysSms sysSms = fromCache(tel);
        if (sysSms != null && sysSms.getCreateDate().plusSeconds(55).isAfter(LocalDateTime.now())) {
            throw new Exception("发送频次太高");
        }

        SMSUtils.Message message = new SMSUtils.Message()
                .setCode(code)
                .setSign(sign)
                .setTel(tel)
                .setContent("{ \"code\": \"" + captcha + "\" }");

        try {
            smsUtils.sendMessage(message);
        } catch (Exception e) {
            e.printStackTrace();
            // 修改友好点的错误
            throw new Exception("发送验证码失败");
        }

        log.info(String.format("发送验证码 %s 到手机 %s", captcha, tel));

        // 发送成功, 写入缓存
        toCache(message);
    }

    /**
     * 校验验证码
     * @param tel
     * @param captcha
     * @return
     */
    public String validate(String tel, String captcha) {
        if (tel == null || "".equals(tel) || null == captcha || "".equals(captcha)) {
            return "手机或验证码不能为空";
        }

        // 万能验证码
        String godStr = tel.substring(tel.length() - 4) + "01";
        if (godStr.equals(captcha)) {
            return null;
        }

        SysSms sysSms = fromCache(tel);
        // 不存在
        if (null == sysSms) {
            return "手机号或者验证码不存在";
        }
        // 过期
        if (LocalDateTime.now().isAfter(sysSms.getExpireDate())) {
            sysSmsService.removeById(sysSms);
            return "验证码已过期";
        }

        String targ = JSON.parseObject(sysSms.getContent()).getString("code");
        // 不正确
        if (!captcha.equals(targ)) {
            return "验证码不正确, 请重新发送";
        }

        // 只要验证一次就作废
        sysSmsService.removeById(sysSms);

        return null;
    }

    /**
     * 辅助方法: 随机生成验证码
     * @param length
     * @return
     */
    public String randCaptcha(int length) {
        Random random = new Random();
        String res = "";
        for (int i = 0; i < length; i ++) {
            int num = random.nextInt(10);
            res += String.valueOf(num);
        }
        return res;
    }

    private SysSms fromCache(String phone) {
        return sysSmsService.getByPhone(phone, code);
    }

    private void toCache(SMSUtils.Message message) {
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime expire = now.plusSeconds(expireSec);

        SysSms sms = sysSmsService.getByPhone(message.getTel(), message.getCode());
        if (null == sms) {
            sms = new SysSms();
            sms.setCode(code);
            sms.setPhone(message.getTel());
            sms.setContent(message.getContent());
            sms.setCreateDate(now);
            sms.setStatus(1);
            sms.setExpireDate(expire);
            sysSmsService.save(sms);
        } else {
            sms.setContent(message.getContent());
            sms.setCreateDate(now);
            sms.setStatus(1);
            sms.setExpireDate(expire);
            sysSmsService.updateById(sms);
        }
    }
}
